
from IPython.core import magic_arguments
from IPython.core.magic import line_magic, cell_magic, line_cell_magic, Magics, magics_class

from ..VERSION import VERSION

from ..core.api import Dsl
from ..core.utils import *

@magics_class
class DslMagics(Magics):

    connection = None
    results_var = "dsl_last_results" # VAR AUTOMATICALLY SET TO LATEST QUERY RESULTS

    
    # NOTE line magics and arguments don't play well together when you need to parse a line content
    # in that case the args parsing throws an error
    # hence better to have a cell magic, and pass only the args in the line

    @line_magic
    @magic_arguments.magic_arguments()
    @magic_arguments.argument('--endpoint',
      help='The query endpoint: default is https://app.dimensions.ai'
    )
    @magic_arguments.argument('--usr',
      help='The account username'
    )
    @magic_arguments.argument('--psw',
      help='The account password'
    )
    @magic_arguments.argument('--env',
      help='The instance name as defined in init.py. Default: live.'
    )
    def dsl_login(self, line):
        """DimCli Magic
        Authenticate with the Dimensions.ai DSL backend. If not args are passed, it assumed you have set up a dsl.ini file (see https://github.com/lambdamusic/dimcli#the-credentials-file).
        Alternatively one can pass auth details explicitly eg
        >>> %dsl_login --user=me@mail.com --password=secret

        """
        args = magic_arguments.parse_argstring(self.dsl_login, line)
        print(args)
        usr = args.usr
        psw = args.psw
        endpoint = args.endpoint or "https://app.dimensions.ai"
        instance = args.env or "live"

        if usr and psw:
            self.connection = Dsl(user=usr, password=psw, endpoint=endpoint, show_results=False)
            print("DimCli %s - Succesfully connected to <%s> (method: manual login)" % (str(VERSION), self.connection._url)) 
        else:
            # try to use local init file
            self.connection = Dsl(instance=instance, show_results=False) 
            print("DimCli %s - Succesfully connected to <%s> (method: dsl.ini file)" % (str(VERSION), self.connection._url))

        # self.connection = Dsl(instance=instance, show_results=False) 
        


    @line_cell_magic
    def dsl_query(self, line, cell=None):
        """DimCli Magic
        Query the Dimensions DSL API with the text passed. 
        """
        if self.connection:
            if cell:
                line = cell
            data = self._handle_query(line)
            self.shell.user_ns[self.results_var] = data
            return data
            
        else:
            print("Please login first: %dsl_login")

    @line_cell_magic
    def dsl_query_loop(self, line, cell=None):
        """DimCli Magic
        Query the Dimensions DSL API with the text passed, looping over all results pages. The final object returns contains all results into a single JSON object.  
        """
        if self.connection:
            if cell:
                line = cell
            data = self._handle_query(line, loop=True)
            self.shell.user_ns[self.results_var] = data
            return data

        else:
            print("Please login first: %dsl_login")


    def _handle_query(self, text, loop=False):
        """main procedure after user input"""
       
        # lazy complete
        text = line_add_lazy_return(text)
        text = line_add_lazy_describe(text)

        # RUN QUERY
        if not loop:
            res = self.connection.query(text)
        else:
            res = self.connection.query_iterative(text)
            return res
        
        # print results
        if "errors" in res.data.keys():
            if "query" in res.data["errors"]:
                print(res.data["errors"]["query"]["header"])
                for key in res.data["errors"]["query"]["details"]:
                    print(key)
            else:
                print(res.data["errors"])
        else:
            print_json_summary(res, text)
            return res


    @line_magic
    def dsl_schema(self, line):
        "Wrapper around the dsl describe function"
        if self.connection:
            return self._handle_query("describe schema")
        else:
            print("Please login first: %dsl_login")


ip = get_ipython()
ip.register_magics(DslMagics)

